/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.beans.session;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;

import kieker.analysis.AnalysisController;
import kieker.analysis.exception.AnalysisConfigurationException;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.MIRepository;
import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.repository.AbstractRepository;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.common.logging.LogImplWebguiLogging;
import kieker.webgui.common.PluginClassLoader;

/**
 * This bean contains an instance of {@link AnalysisController} and supplies methods to control the instance.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@SessionScoped
public class AnalysisControllerBean {
	/**
	 * The logger within this class.
	 */
	private static final Log LOG = LogFactory.getLog(AnalysisControllerBean.class);
	/**
	 * The controller instance stored within this bean.
	 */
	private AnalysisController controller;
    private HashMap<MIPlugin, AbstractPlugin> pluginMap;
    private HashMap<MIRepository, AbstractRepository> repositoryMap;

	/**
	 * Creates a new instance of this class.
	 */
	public AnalysisControllerBean() {
		// No code necessary.
	}

	/**
	 * Delivers the controller stored within this bean.
	 * 
	 * @return The current controller in this bean.
	 */
	public AnalysisController getController() {
		return this.controller;
	}

	/**
	 * Sets the controller stored within this bean to a new value.
	 * 
	 * @param controller
	 *            The new controller.
	 */
	public void setController(final AnalysisController controller) {
		this.controller = controller;
	}

	/**
	 * This method tries to instantiate a new controller using the given project. If the project is null, nothing happens.
	 * 
	 * @param mProject
	 *            The project used to create a new analysis controller.
	 */
	public void instantiate(final MIProject mProject) {
		if (mProject != null) {
			try {
				// Try to create the controller.
                final AnalysisController.AnalysisControllerWithMapping controllerAndMapping = AnalysisController.createAnalysisController(mProject, PluginClassLoader.getInstance());
				// Extract everything that is necessary.
                this.controller = controllerAndMapping.getController();
                this.pluginMap = controllerAndMapping.getPluginMap();
                this.repositoryMap = controllerAndMapping.getRepositoryMap();
			} catch (final NullPointerException ex) {
				AnalysisControllerBean.LOG.error("Could not create analysis controller.", ex);
			} catch (final AnalysisConfigurationException ex) {
				AnalysisControllerBean.LOG.error("Could not create analysis controller.", ex);
			}
		}
	}

	/**
	 * Starts asynchronously the analysis controller within this bean.
	 */
	public void start() {
		final Thread thread = new Thread() {
			@Override
			public void run() {
				try {
					AnalysisControllerBean.this.controller.run();
				} catch (final IllegalStateException ex) {
					AnalysisControllerBean.LOG.error("Could not start the analysis controller.", ex);
				} catch (final AnalysisConfigurationException ex) {
					AnalysisControllerBean.LOG.error("Could not start the analysis controller.", ex);
				}
			}
		};
		thread.start();
	}

	/**
	 * Stops asynchronously the analysis controller within this bean.
	 */
	public void stop() {
		final Thread thread = new Thread() {
			@Override
			public void run() {
				AnalysisControllerBean.this.controller.terminate();
			}
		};
		thread.start();
	}

	/**
	 * This method delivers all log entries of the class {@link AnalysisController}.
	 * 
	 * @return All log entries of the analysis controller.
	 */
	public String[] getLogEntries() {
		return LogImplWebguiLogging.getEntries(AnalysisController.class.getName());
	}
}
